part of redstone.rethinkdb;

app.RedstonePlugin rethinkPlugin(RethinkDbManager db, {String pattern: r'/.*'}) {

  return (app.Manager manager) {

    if (db == null)
      return;

    manager.addInterceptor(new app.Interceptor.conf(pattern), "database connection manager", (injector) async {

      RethinkConnection conn = await db.getConnection();
      app.request.attributes["dbConn"] = conn.conn;

      app.chain.next(() {
        db.closeConnection(conn, error: app.chain.error);
      });

    });
  };
}

setupRethink(ConfigRethink config) async {

  Rethinkdb r = new Rethinkdb();
  Connection conn = await r.connect(host: config.host, db: config.database, port: config.port);

  if (!await r.dbList().contains(config.database).run(conn)) {
    await r.dbCreate(config.database).run(conn);
    print('Created db: ${config.database}');
  }

  List tables = await r.db(config.database).tableList().run(conn);
  for (var table in config.tables) {
    if (!tables.contains(table)) {
      await r.tableCreate(table).run(conn);
      print('Created table: $table');
    }
  }

  conn.close();
}