part of redstone.rethinkdb;

class RethinkServices<T> extends Table {
  final Rethinkdb r = new Rethinkdb();
  final String tableName;
  Connection _conn;

  RethinkServices(String tableName):
    this.tableName = tableName,
    super(tableName);

  RethinkServices.fromConnection(String tableName, Connection _conn):
    this._conn = _conn,
    this.tableName = tableName,
    super(tableName);

  Connection get conn {
    if (_conn != null)
      return _conn;

    var rConn = app.request.attributes.dbConn;
    return rConn is RethinkConnection? rConn.conn: rConn;
  }
  Table get table => r.table(tableName);

  Insert insertType(T record, {options}) {
    return insert(encode(record), options);
  }

  Future<Map> insertNow(T record, {options, global_optargs}) async {
    return insertType(record, options: options).run(conn, global_optargs);
  }

  Future<T> getNow (String id, {global_optargs}) async
  {
    var record = await get(id).run(conn, global_optargs);
    return record != null? encode(record): null;
  }

  Update updateTyped (String id, T record, {options})
    => get(id).update(encode(record), options);

  Future updateNow (String id, T record, {options, global_optargs})
    => updateTyped(id, record, options: options).run(conn, global_optargs);

  Future deleteNow (String id, {options, global_optargs})
    => get(id).delete(options).run(conn, global_optargs);

  Future<T> findOne (condition) async {
    Cursor cursor = await filter(condition).limit(1).run(conn);
    var result = (await cursor.toArray()).first;
    return result != null? decode(result, T): null;
  }
}
